/*
 *    SOAPCommunicator.java
 *    Copyright (C) 2002 New Zealand Digital Library, http://www.nzdl.org
 *
 *    This program is free software; you can redistribute it and/or modify
 *    it under the terms of the GNU General Public License as published by
 *    the Free Software Foundation; either version 2 of the License, or
 *    (at your option) any later version.
 *
 *    This program is distributed in the hope that it will be useful,
 *    but WITHOUT ANY WARRANTY; without even the implied warranty of
 *    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *    GNU General Public License for more details.
 *
 *    You should have received a copy of the GNU General Public License
 *    along with this program; if not, write to the Free Software
 *    Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */
package org.greenstone.gsdl3.comms;

import org.greenstone.gsdl3.util.*;
import org.greenstone.gsdl3.core.*;

// classes needed for SOAP stuff
import java.net.URL;
import java.net.MalformedURLException;
import java.util.Vector;
import org.apache.soap.SOAPException;
import org.apache.soap.Fault;
import org.apache.soap.Constants;
import org.apache.soap.rpc.Call;
import org.apache.soap.rpc.Parameter;
import org.apache.soap.rpc.Response;

import org.w3c.dom.Node; 
import org.w3c.dom.NodeList; 
import org.w3c.dom.Document; 
import org.w3c.dom.Element; 

/*
 * The Client side of a SOAP Connection
 *
 * @author <a href="mailto:kjdon@cs.waikato.ac.nz">Katherine Don</a>
 * @version $Revision: 3505 $
 * @see <a href="http://www.w3.org/TR/SOAP/">Simple Object Access Protocol (SOAP) 1.1 </a>
 */
public class SOAPCommunicator
    extends Communicator {

    /** address of site to connect to */
    protected String remote_site_address_=null;

    /** the call object that does the SOAP talking */
    private Call call_ = null;

  /** The no-args constructor */
    public SOAPCommunicator() {
	
	call_ = new Call();
	// Set Encoding Style to standard SOAP encoding
	//call_.setEncodingStyleURI(Constants.NS_URI_SOAP_ENC);
	// set Encoding Style to use literal XML
	call_.setEncodingStyleURI(Constants.NS_URI_LITERAL_XML);
    }
        
    public boolean configure(Element site_elem) {

	String type = site_elem.getAttribute("type");
	if (!type.equals("soap")) {
	    System.err.println("SOAPCommunicator: wrong type of site");
	    return false;
	}
	remote_site_name_ = site_elem.getAttribute("name");
	remote_site_address_ = site_elem.getAttribute("address");
	if (remote_site_name_==null || remote_site_name_.equals("")) {
	    System.err.println("SOAPCommunicator: must have name attribute in site element");
	    return false;
	}
	if (remote_site_address_==null || remote_site_address_.equals("")) {
	    System.err.println("SOAPCommunicator: must have address attribute in site element");
	    return false;
	}
	call_.setTargetObjectURI(remote_site_name_);
	call_.setMethodName("process");

	return true;
    }


    // put this into Communicator??
    public Element process(Element xml_in) {
	// needs to catch SOAPException, MalformedURLException
	try {
	    // add the message encloser back on, and edit the to field
	    Document newdoc = converter_.newDOM();
	    Element top = newdoc.createElement("message");
	    top.appendChild(newdoc.importNode(xml_in, true));
	    //newdoc.appendChild(top);
	    NodeList requests = top.getElementsByTagName("request");
	    if (requests.getLength()==0) {
		// no requests
		return null;
	    }
	    for (int i=0;i<requests.getLength(); i++) {
		Element e = (Element)requests.item(i);
		String to = e.getAttribute("to");
		to = GSPath.removeFirstLink(to); // remove the server name, should check that it is present
		e.setAttribute("to", to);
		String from = e.getAttribute("from");
		from = GSPath.appendLink(from, local_site_name_);
		e.setAttribute("from", from);
	    }
	    
	    // Set Method Parameters

	    // dont use Strings anymore
	    //String converted_request = converter_.getString(top);
	    //System.out.println("converted request = "+converted_request);
	    //  
	    //Parameter param = new Parameter("xml_in", String.class,
	    //converted_request, Constants.NS_URI_SOAP_ENC);

	    
	    // use  straight xml
	    Parameter param = new Parameter("xml_in", 
					    org.w3c.dom.Element.class,
					    top, 
					    Constants.NS_URI_LITERAL_XML);
					    
	    Vector param_list = new Vector ();
	    param_list.addElement (param);
	    call_.setParams (param_list);
	    
	    //  Set the URL for the Web Service
	    URL url = new URL(remote_site_address_);
	    
	    // Invoke the Service
	    Response resp = call_.invoke (url, "");
	    
	    // Check for Faults
	    if (!resp.generatedFault()) {
	    // Extract Return value
		Parameter result = resp.getReturnValue ();
		//String res = (String) result.getValue();
		//Element result_node = converter_.getDOM(res).getDocumentElement();
		Element result_node = (Element) result.getValue();

		// have to modify the from field

		NodeList responses = result_node.getElementsByTagName("response");
		for (int i=0;i<responses.getLength(); i++) {
		    Element e = (Element)responses.item(i);
		    String from = e.getAttribute("from");
		    from = GSPath.prependLink(from, remote_site_name_);
		    e.setAttribute("from", from);
		}

		return result_node;
	    }
	    else {
		//  Extract Fault Code and String
		Fault f = resp.getFault();
		String faultCode = f.getFaultCode();
		String faultString = f.getFaultString();
		System.err.println("Fault Occurred (details follow):");
		System.err.println("Fault Code:  "+faultCode);
		System.err.println("Fault String:  "+faultString);
		return converter_.getDOM("<message>Fault Occurred.  No greeting for you!</message>").getDocumentElement();
	    }
	} catch (SOAPException e) {
	    System.out.println("SOAP error:"+e.getMessage());
	    return null;
	} catch (MalformedURLException e) {
	    System.out.println("URL error:"+e.getMessage());
	    return null;
	}
    }
    
    public String process(String xml_in) {
	Element e = converter_.getDOM(xml_in).getDocumentElement();
	Node result = process(e);
	return converter_.getString(result);	
    }
}








