package org.greenstone.gsdl3.core;

import org.greenstone.gsdl3.util.*;
import org.greenstone.gsdl3.action.*;
// XML classes
import org.w3c.dom.Node; 
import org.w3c.dom.NodeList; 
import org.w3c.dom.Document; 
import org.w3c.dom.Element; 

// other java classes
import java.io.File;
import java.util.HashMap;

public class Receptionist implements ModuleInterface {

    /** the set up variables */
    protected ConfigVars config_=null;

    /** the cgi args converter */
    protected CGIArgConverter cgi_=null;

    /** container Document to create XML Nodes */
    protected Document doc_=null; 
    /** a converter class to parse XML and create Docs */
    protected XMLConverter converter_=null;

    /** the message router that it talks to */
    protected ModuleInterface mr_=null;

    /** the list of actions */
    protected HashMap action_map_=null;

    protected Translate translation_=null;
    public Receptionist() {
	converter_ = new XMLConverter();
	doc_ = converter_.newDOM();
	action_map_= new HashMap();
	translation_ = new Translate();
	cgi_ = new CGIArgConverter();
    }
 
    /** the set up vars must be set before configure called*/
    public void setConfigVars(ConfigVars config) {
	config_ = config;
	translation_.setConfigVars(config);
    }
    /** sets the message router  - it should already be created and 
     * configured before  being passed to the receptionist*/
    public void setMessageRouter(ModuleInterface m) {
       mr_ = m;
    }

    public boolean configure() {

	if (config_==null) {
	    System.err.println("You must set the config variables before calling configure");
	    return false;
	}
	if (mr_==null) {	   
	    System.err.println("You must set  the mr  before calling configure");
	    return false;
	}

	// for now, assume default is en, so load that
	translation_.setDefaultLanguage("en");
	// for now, just statically add in the actions
	// actions may disappear if xslt is all that is needed
	// or they should be dynamically loaded
	Action a = new PageAction();
	a.setConfigVars(config_);
	a.setMessageRouter(mr_);
	a.setCGIConverter(cgi_);
	a.configure();
	action_map_.put("p", a);

	a = new QueryAction();
	a.setConfigVars(config_);
	a.setCGIConverter(cgi_);
	a.setMessageRouter(mr_);
	a.configure();
	action_map_.put("q", a);

	a = new BrowseAction();
	a.setConfigVars(config_);
	a.setCGIConverter(cgi_);
	a.setMessageRouter(mr_);
	a.configure();
	action_map_.put("b", a);

	a = new ResourceAction();
	a.setConfigVars(config_);
	a.setCGIConverter(cgi_);
	a.setMessageRouter(mr_);
	a.configure();
	action_map_.put("r", a);

	a = new AppletAction();
	a.setConfigVars(config_);
	a.setCGIConverter(cgi_);
	a.setMessageRouter(mr_);
	a.configure();
	action_map_.put("a", a);
	return true;
    }

    public String process(String xml_in) {

	Document message_doc = converter_.getDOM(xml_in);
	Element message = message_doc.getDocumentElement();
	// there should only be one request per message
	Element request = (Element)GSXML.getChildByTagName(message, "request");
	if (request == null) {
	    return GSHTML.errorPage("no request in the message");
	}

	String type = request.getAttribute("type"); // returns "" if no att of this name
	if (!type.equals("action")) {
	    return GSHTML.errorPage("wrong type in request - should be action");
	}	

	String action = request.getAttribute("action");
	if (action.equals("")) {
	    return GSHTML.errorPage(" no action specified");
	}
	// pass to appropriate action
	
	Action a = (Action)action_map_.get(action);
	if (a==null) {
	    // try to load a new action
	    try {
		String action_name = action.substring(0,1).toUpperCase()+action.substring(1)+"Action";
		System.out.println("new action name="+action_name);
		Action ac = (Action)Class.forName("org.greenstone.gsdl3.action."+action_name).newInstance();
		ac.setConfigVars(config_);
		ac.setMessageRouter(mr_);
		ac.setCGIConverter(cgi_);
		ac.configure();
		action_map_.put(action, ac);
		a = ac;
	    } catch (Exception e) {
		
		return GSHTML.errorPage("no action for "+action+", and couldn't create a new one: error message:"+e.getMessage());
	    }
	}

	// add in the translation bit - should we do it here? or give the actions access to the translation object?
	String lang = message.getAttribute("lang");
	Element trans = translation_.getLanguageTree(lang);
	message.appendChild(message_doc.importNode(trans, true));

	// add in the system vars bit
	message.appendChild(message_doc.importNode(config_.config_xml_, true));

	String page = a.process(message);
	return page;
    }

    public Element process(Element xml_in) {
	return xml_in; // for now

    }


}
